import requests
import subprocess
from bs4 import BeautifulSoup
from urllib.parse import urljoin, urlparse

# Function to scrape all link tags and all src attributes from the page 
def get_all_sources(url):
    response = requests.get(url)

    if response.status_code != 200:
        print(f"[-] Failed to request the website. Status code: {response.status_code}")
        return set()

    soup = BeautifulSoup(response.text, 'html.parser')
    all_sources = set()

    # Extract all CSS links (href)
    for link_tag in soup.find_all('link', {'href': True}):
        href = link_tag.get('href')
        full_url = urljoin(url, href)
        all_sources.add(full_url)

    # Extract all source attributes
    for tag in soup.find_all(True, {'src': True}):
        src = tag.get('src')
        full_url = urljoin(url, src)
        all_sources.add(full_url)

    return all_sources

if __name__ == "__main__":
    url_input = input("[>] Provide URL: ")

    # Extract domain
    parsed_url = urlparse(url_input)
    domain = parsed_url.netloc

    sources = get_all_sources(url_input)

    # Add the original URL to the set of sources
    sources.add(url_input)

    # Filter internal sources (those that contain the original URL)
    internal_sources = {src for src in sources if domain in urlparse(src).netloc}

    if internal_sources:
        print("[+] Found sources:")
        for src in internal_sources:
            print(src)
        print("[+] Starting multi-threaded downloader")
        subprocess.run(["./url_scraper/target/release/url_scraper"] + list(internal_sources))
    else:
        print("[-] No files found")
